package org.apache.turbine.flux.tools;

/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and 
 *    "Apache Turbine" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For 
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    "Apache Turbine", nor may "Apache" appear in their name, without 
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

import java.util.Vector;
import java.util.Iterator;

import org.apache.velocity.context.Context;

import org.apache.turbine.om.security.Group;
import org.apache.turbine.om.security.Role;
import org.apache.turbine.om.security.Permission;
import org.apache.turbine.om.security.User;
import org.apache.turbine.util.security.AccessControlList;
import org.apache.turbine.om.security.peer.TurbineUserPeer;
import org.apache.turbine.util.RunData;
import org.apache.turbine.util.db.Criteria;
import org.apache.turbine.util.template.SelectorBox;
import org.apache.turbine.services.security.TurbineSecurity;
import org.apache.turbine.services.pull.ApplicationTool;
import org.apache.turbine.util.pool.Recyclable;
import org.apache.turbine.util.Log;
//import org.apache.turbine.util.ParameterParser;
//import org.apache.turbine.util.ValueParser;


import org.apache.turbine.flux.modules.screens.FluxScreen;

/**
 * The pull api for flux templates
 *
 * @author <a href="mailto:jmcnally@apache.org">John D. McNally</a>
 */
public class FluxTool 
    implements Recyclable, ApplicationTool
{
    /** the object containing request specific data */
    private RunData data;

    /**
     * A Group object for use within the Flux API.
     */
    private Group group = null;

    /**
     * A Issue object for use within the Flux API.
     */
    private Role role = null;

    /**
     * A Permission object for use within the Flux API.
     */
    private Permission permission = null;

    /**
     * A User object for use within the Flux API.
     */
    private User user = null;

    
    public void init(Object data)
    {
        this.data = (RunData)data;
    }

    /**
     * nulls out the issue and user objects
     */
    public void refresh()
    {
        // do not need since it is a request tool
    }

    /**
     * Constructor does initialization stuff
     */    
    public FluxTool()
    {
    }


    public Group getGroup()
        throws Exception
    {
        if (group == null)
        {
            String name = data.getParameters().getString("name"); 
            if ( name == null || name.length() == 0)
            {
                group = TurbineSecurity.getNewGroup(null);
            }
            else 
            {
                group = TurbineSecurity.getGroup(name);
            }
        }
        return group;
    }

    public String getMode()
    {
        return data.getParameters().getString("mode");
    }


    public Group[] getGroups()
        throws Exception
    { 
        return TurbineSecurity.getAllGroups().getGroupsArray();
    }

    public Role getRole()
        throws Exception
    {
        if (role == null)
        {
            String name = data.getParameters().getString("name"); 
            if ( name == null || name.length() == 0)
            {
                role = TurbineSecurity.getNewRole(null);  
            }
            else 
            {
                role = TurbineSecurity.getRole(name);
            }
        }
        return role;
    }

    /**
     */
    public Role[] getRoles()
        throws Exception
    {
        Criteria criteria = new Criteria();
        System.out.println("Getting roles");
        System.out.println("roles.size=" + TurbineSecurity.getRoles(criteria).getRolesArray().length);
        return TurbineSecurity.getRoles(criteria).getRolesArray();
    }

    public Permission getPermission()
        throws Exception
    {
        if (permission == null)
        {
            String name = data.getParameters().getString("name"); 
            if ( name == null || name.length() == 0)
            {
                permission = TurbineSecurity.getNewPermission(null);  
            }
            else 
            {
                permission = TurbineSecurity.getPermission(name);
            }
        }
        return permission;
    }

    /**
     * Get all permissions.
     */
    public Permission[] getPermissions()
        throws Exception
    {
        return TurbineSecurity.getAllPermissions().getPermissionsArray();
    }

    public User getUser()
        throws Exception
    {
        if (user == null)
        {
            String name = data.getParameters().getString("username"); 
            if ( name == null || name.length() == 0)
            {
                user = TurbineSecurity.getUserInstance();  
            }
            else 
            {
                user = TurbineSecurity.getUser(name);
            }
        }
        return user;
    }

    public AccessControlList getACL()
        throws Exception
    {
        return  TurbineSecurity.getACL(getUser());
    }



    /**
     */
    public SelectorBox getFieldList() throws Exception
    {
        Object[] names = {"username", "firstname", "middlename", "lastname"}; 
        Object[] values = 
            { "Username", "First Name", "Middle Name", "Last Name" };
        return  new SelectorBox("fieldList", names, values);
    }

    /**
     */
    public SelectorBox getUserFieldList() 
        throws Exception
    {
        /**
         * This is a tie to the DB implementation
         * something should be added the pluggable pieces
         * to allow decent parameterized searching.
         */
    
        Object[] names = 
        { 
            TurbineUserPeer.USERNAME, 
            TurbineUserPeer.FIRST_NAME, 
            TurbineUserPeer.LAST_NAME 
        }; 
        
        Object[] values = 
        { 
            "User Name" , 
            "First Name", 
            "Last Name" 
        };

        return  new SelectorBox("fieldList", names, values);
    }

    /**
     * Select all the users and place them in an array
     * that can be used within the UserList.vm template.
     */
    public User[] getUsers() 
        throws Exception
    {
        Criteria criteria = new Criteria();
        
        String fieldList = data.getParameters().getString("fieldList");
        
        if (fieldList != null)
        {
            /*
             * This is completely database centric.
             */
            String searchField = data.getParameters().getString("searchField");
            
            /*
             * Does this tie this admin app to a database
             * implementation? And our implementation at
             * that?
             */
            
            /*
             * How the hell do you make a where name like 'j%'
             * type statement with Criteria?
             */
            criteria.add(fieldList, (Object) searchField, Criteria.LIKE);
        }
        
        return TurbineSecurity.getUsers(criteria);
    }

    // ****************** Recyclable implementation ************************

    private boolean disposed;

    /**
     * Recycles the object for a new client. Recycle methods with
     * parameters must be added to implementing object and they will be
     * automatically called by pool implementations when the object is
     * taken from the pool for a new client. The parameters must
     * correspond to the parameters of the constructors of the object.
     * For new objects, constructors can call their corresponding recycle
     * methods whenever applicable.
     * The recycle methods must call their super.
     */
    public void recycle()
    {
        disposed = false;
    }

    /**
     * Disposes the object after use. The method is called
     * when the object is returned to its pool.
     * The dispose method must call its super.
     */
    public void dispose()
    {
        data = null;
        user = null;
        group = null;
        role = null;
        permission = null;

        disposed = true;
    }

    /**
     * Checks whether the recyclable has been disposed.
     * @return true, if the recyclable is disposed.
     */
    public boolean isDisposed()
    {
        return disposed;
    }


}



    /* *
     * A Attribute object for use within the Scarab API.
     * /
    public void setAttribute (Attribute attribute)
    {
        this.attribute = attribute;
    }

    private IntakeTool getIntakeTool()
    {
        return (IntakeTool)TurbineVelocity.getContext(data)
            .get(ScarabConstants.INTAKE_TOOL);
    }


    /* *
     * A Attribute object for use within the Scarab API.
     * /
    public void setAttributeOption (AttributeOption option)
    {
        this.attributeOption = option;
    }

    /* *
     * A Attribute object for use within the Scarab API.
     * /
    public AttributeOption getAttributeOption()
        throws Exception
    {
try{
        if (attributeOption == null)
        {
            String optId = data.getParameters()
                .getString("currentAttributeOption"); 
            if ( optId == null )
            {
                attributeOption = new AttributeOption();                
            }
            else 
            {
                attributeOption = AttributeOptionPeer
                    .retrieveByPK(new NumberKey(optId));
            }
        }
}catch(Exception e){e.printStackTrace();}
        return attributeOption;
    }

    /* *
     * A User object for use within the Scarab API.
     * /
    public void setUser (ScarabUser user)
    {
        this.user = user;
    }

    /* *
     * A User object for use within the Scarab API.
     * /
    public ScarabUser getUser()
    {
        if (user == null)
            this.user = (ScarabUser)data.getUser();
        return this.user;
    }

    /* *
     * A Attribute object for use within the Scarab API.
     * /
    public Attribute getAttribute()
     throws Exception
    {
try{
        if (attribute == null)
        {
            String attId = getIntakeTool()
                .get("Attribute", IntakeTool.DEFAULT_KEY).get("Id").toString();
            if ( attId == null || attId.length() == 0 )
            {
                attribute = Attribute.getInstance();
            }
            else 
            {
                attribute = Attribute.getInstance(new NumberKey(attId));
            }
        }        
}catch(Exception e){e.printStackTrace();}
        return attribute;
 
   }
    /* *
     * A Module object for use within the Scarab API.
     * /
    public void setModule(Module module)
    {
        this.module = module;
    }

    /* *
     * Get an Module object. 
     *
     * @return a <code>Module</code> value
     * /
    public Module getModule()
     throws Exception
    {
      try{
            String modId = getIntakeTool()
                .get("Module", IntakeTool.DEFAULT_KEY).get("Id").toString();
            if ( modId == null || modId.length() == 0 )
            {
                module = ModuleManager.getInstance();
            }
            else 
            {
                module = ModuleManager.getInstance(new NumberKey(modId));
            }
      }catch(Exception e){e.printStackTrace();}
        return module;
 
   }
    /* *
     * Get an RModuleAttribute object. 
     *
     * @return a <code>Module</code> value
     * /
    public RModuleAttribute getRModuleAttribute()
        throws Exception
    {
        RModuleAttribute rma = null;
      try{
            ComboKey rModAttId = (ComboKey)getIntakeTool()
                .get("RModuleAttribute", IntakeTool.DEFAULT_KEY)
                .get("Id").getValue();
            if ( rModAttId == null )
            {
                NumberKey attId = (NumberKey)getIntakeTool()
                    .get("Attribute", IntakeTool.DEFAULT_KEY)
                    .get("Id").getValue();
                if ( attId != null && getUser().getCurrentModule() != null )
                {
                    NumberKey[] nka = {attId, 
                        getUser().getCurrentModule().getModuleId()};
                    rma = RModuleAttributePeer.retrieveByPK(new ComboKey(nka));
                }
                else 
                {
                    rma = new RModuleAttribute();
                }
            }
            else 
            {
                rma = RModuleAttributePeer.retrieveByPK(rModAttId);
            }
      }catch(Exception e){e.printStackTrace();}
        return rma;
 
   }

    

    /* *
     * Get a specific module by key value.
     *
     * @param key a <code>String</code> value
     * @return a <code>Module</code> value
     * /
    public Module getModule(String key) throws Exception
    {
        return ModulePeer.retrieveByPK(new NumberKey(key));
    }


    /* *
     * A Issue object for use within the Scarab API.
     * /
    public void setIssue(Issue issue)
    {
        this.issue = issue;
    }

    /* *
     * Get an Issue object. If it is the first time calling,
     * it will be a new blank issue object.
     *
     * @return a <code>Issue</code> value
     * /
    public Issue getIssue()
    {
        if (issue == null)
            this.issue = new Issue();
        return this.issue;
    }

    /* *
     * Get a new SearchIssue object. 
     *
     * @return a <code>Issue</code> value
     * /
    public Issue getSearch()
        throws Exception
    {
        IssueSearch search = new IssueSearch();
        search.setModule(((ScarabUser)getUser()).getCurrentModule());
        return search;
    }

    /* *
     * The id may be a primary key or an issue id.
     *
     * @param key a <code>String</code> value
     * @return a <code>Issue</code> value
     * /
    public Issue getIssue(String key)
        throws Exception
    {
        Issue issue = null;
        try
        {
            issue = IssuePeer.retrieveByPK(new NumberKey(key));
        }
        catch (RuntimeException re)
        {
            // was not a primary key, try fid
            Issue.FederatedId fid = new Issue.FederatedId(key);
            if ( fid.getDomain() == null ) 
            {
                // handle null (always null right now)
            }
            issue = Issue.getIssueById(fid);
        }

        return issue;
    }
    */
